@@ -25,13 +25,11 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"math/big"
 	"os"
 	"os/signal"
 	"path/filepath"
 	"runtime"
-	"sort"
 	"strings"
 	"time"
 
@@ -56,7 +54,7 @@ import (
 	"github.com/ethereum/go-ethereum/signer/storage"
 	"github.com/mattn/go-colorable"
 	"github.com/mattn/go-isatty"
-	"gopkg.in/urfave/cli.v1"
+	"github.com/urfave/cli/v2"
 )
 
 const legalWarning = `
@@ -74,70 +72,70 @@ PURPOSE. See the GNU General Public License for more details.
 `
 
 var (
-	logLevelFlag = cli.IntFlag{
+	logLevelFlag = &cli.IntFlag{
 		Name:  "loglevel",
 		Value: 4,
 		Usage: "log level to emit to the screen",
 	}
-	advancedMode = cli.BoolFlag{
+	advancedMode = &cli.BoolFlag{
 		Name:  "advanced",
 		Usage: "If enabled, issues warnings instead of rejections for suspicious requests. Default off",
 	}
-	acceptFlag = cli.BoolFlag{
+	acceptFlag = &cli.BoolFlag{
 		Name:  "suppress-bootwarn",
 		Usage: "If set, does not show the warning during boot",
 	}
-	keystoreFlag = cli.StringFlag{
+	keystoreFlag = &cli.StringFlag{
 		Name:  "keystore",
 		Value: filepath.Join(node.DefaultDataDir(), "keystore"),
 		Usage: "Directory for the keystore",
 	}
-	configdirFlag = cli.StringFlag{
+	configdirFlag = &cli.StringFlag{
 		Name:  "configdir",
 		Value: DefaultConfigDir(),
 		Usage: "Directory for Clef configuration",
 	}
-	chainIdFlag = cli.Int64Flag{
+	chainIdFlag = &cli.Int64Flag{
 		Name:  "chainid",
 		Value: params.MainnetChainConfig.ChainID.Int64(),
 		Usage: "Chain id to use for signing (1=mainnet, 3=Ropsten, 4=Rinkeby, 5=Goerli)",
 	}
-	rpcPortFlag = cli.IntFlag{
-		Name:  "http.port",
-		Usage: "HTTP-RPC server listening port",
-		Value: node.DefaultHTTPPort + 5,
+	rpcPortFlag = &cli.IntFlag{
+		Name:     "http.port",
+		Usage:    "HTTP-RPC server listening port",
+		Value:    node.DefaultHTTPPort + 5,
+		Category: flags.APICategory,
 	}
-	signerSecretFlag = cli.StringFlag{
+	signerSecretFlag = &cli.StringFlag{
 		Name:  "signersecret",
 		Usage: "A file containing the (encrypted) master seed to encrypt Clef data, e.g. keystore credentials and ruleset hash",
 	}
-	customDBFlag = cli.StringFlag{
+	customDBFlag = &cli.StringFlag{
 		Name:  "4bytedb-custom",
 		Usage: "File used for writing new 4byte-identifiers submitted via API",
 		Value: "./4byte-custom.json",
 	}
-	auditLogFlag = cli.StringFlag{
+	auditLogFlag = &cli.StringFlag{
 		Name:  "auditlog",
 		Usage: "File used to emit audit logs. Set to \"\" to disable",
 		Value: "audit.log",
 	}
-	ruleFlag = cli.StringFlag{
+	ruleFlag = &cli.StringFlag{
 		Name:  "rules",
 		Usage: "Path to the rule file to auto-authorize requests with",
 	}
-	stdiouiFlag = cli.BoolFlag{
+	stdiouiFlag = &cli.BoolFlag{
 		Name: "stdio-ui",
 		Usage: "Use STDIN/STDOUT as a channel for an external UI. " +
 			"This means that an STDIN/STDOUT is used for RPC-communication with a e.g. a graphical user " +
 			"interface, and can be used when Clef is started by an external process.",
 	}
-	testFlag = cli.BoolFlag{
+	testFlag = &cli.BoolFlag{
 		Name:  "stdio-ui-test",
 		Usage: "Mechanism to test interface between Clef and UI. Requires 'stdio-ui'.",
 	}
-	app         = cli.NewApp()
-	initCommand = cli.Command{
-		Action:    utils.MigrateFlags(initializeSecrets),
+	initCommand = &cli.Command{
+		Action:    initializeSecrets,
 		Name:      "init",
 		Usage:     "Initialize the signer, generate secret storage",
 		ArgsUsage: "",
@@ -149,8 +147,8 @@ var (
 The init command generates a master seed which Clef can use to store credentials and data needed for
 the rule-engine to work.`,
 	}
-	attestCommand = cli.Command{
-		Action:    utils.MigrateFlags(attestFile),
+	attestCommand = &cli.Command{
+		Action:    attestFile,
 		Name:      "attest",
 		Usage:     "Attest that a js-file is to be used",
 		ArgsUsage: "<sha256sum>",
@@ -166,8 +164,8 @@ incoming requests.
 Whenever you make an edit to the rule file, you need to use attestation to tell
 Clef that the file is 'safe' to execute.`,
 	}
-	setCredentialCommand = cli.Command{
-		Action:    utils.MigrateFlags(setCredential),
+	setCredentialCommand = &cli.Command{
+		Action:    setCredential,
 		Name:      "setpw",
 		Usage:     "Store a credential for a keystore file",
 		ArgsUsage: "<address>",
@@ -179,8 +177,8 @@ Clef that the file is 'safe' to execute.`,
 		Description: `
 The setpw command stores a password for a given address (keyfile).
 `}
-	delCredentialCommand = cli.Command{
-		Action:    utils.MigrateFlags(removeCredential),
+	delCredentialCommand = &cli.Command{
+		Action:    removeCredential,
 		Name:      "delpw",
 		Usage:     "Remove a credential for a keystore file",
 		ArgsUsage: "<address>",
@@ -192,8 +190,8 @@ The setpw command stores a password for a given address (keyfile).
 		Description: `
 The delpw command removes a password for a given address (keyfile).
 `}
-	newAccountCommand = cli.Command{
-		Action:    utils.MigrateFlags(newAccount),
+	newAccountCommand = &cli.Command{
+		Action:    newAccount,
 		Name:      "newaccount",
 		Usage:     "Create a new account",
 		ArgsUsage: "",
@@ -208,7 +206,7 @@ The newaccount command creates a new keystore-backed account. It is a convenienc
 which can be used in lieu of an external UI.`,
 	}
 
-	gendocCommand = cli.Command{
+	gendocCommand = &cli.Command{
 		Action: GenDoc,
 		Name:   "gendoc",
 		Usage:  "Generate documentation about json-rpc format",
@@ -217,39 +215,16 @@ The gendoc generates example structures of the json-rpc communication types.
 `}
 )
 
-// AppHelpFlagGroups is the application flags, grouped by functionality.
-var AppHelpFlagGroups = []flags.FlagGroup{
-	{
-		Name: "FLAGS",
-		Flags: []cli.Flag{
-			logLevelFlag,
-			keystoreFlag,
-			configdirFlag,
-			chainIdFlag,
-			utils.LightKDFFlag,
-			utils.NoUSBFlag,
-			utils.SmartCardDaemonPathFlag,
-			utils.HTTPListenAddrFlag,
-			utils.HTTPVirtualHostsFlag,
-			utils.IPCDisabledFlag,
-			utils.IPCPathFlag,
-			utils.HTTPEnabledFlag,
-			rpcPortFlag,
-			signerSecretFlag,
-			customDBFlag,
-			auditLogFlag,
-			ruleFlag,
-			stdiouiFlag,
-			testFlag,
-			advancedMode,
-			acceptFlag,
-		},
-	},
-}
+var (
+	// Git SHA1 commit hash of the release (set via linker flags)
+	gitCommit = ""
+	gitDate   = ""
+
+	app = flags.NewApp(gitCommit, gitDate, "Manage Ethereum account operations")
+)
 
 func init() {
 	app.Name = "Clef"
-	app.Usage = "Manage Ethereum account operations"
 	app.Flags = []cli.Flag{
 		logLevelFlag,
 		keystoreFlag,
@@ -274,46 +249,12 @@ func init() {
 		acceptFlag,
 	}
 	app.Action = signer
-	app.Commands = []cli.Command{initCommand,
+	app.Commands = []*cli.Command{initCommand,
 		attestCommand,
 		setCredentialCommand,
 		delCredentialCommand,
 		newAccountCommand,
-		gendocCommand}
-	cli.CommandHelpTemplate = flags.CommandHelpTemplate
-	// Override the default app help template
-	cli.AppHelpTemplate = flags.ClefAppHelpTemplate
-
-	// Override the default app help printer, but only for the global app help
-	originalHelpPrinter := cli.HelpPrinter
-	cli.HelpPrinter = func(w io.Writer, tmpl string, data interface{}) {
-		if tmpl == flags.ClefAppHelpTemplate {
-			// Render out custom usage screen
-			originalHelpPrinter(w, tmpl, flags.HelpData{App: data, FlagGroups: AppHelpFlagGroups})
-		} else if tmpl == flags.CommandHelpTemplate {
-			// Iterate over all command specific flags and categorize them
-			categorized := make(map[string][]cli.Flag)
-			for _, flag := range data.(cli.Command).Flags {
-				if _, ok := categorized[flag.String()]; !ok {
-					categorized[flags.FlagCategory(flag, AppHelpFlagGroups)] = append(categorized[flags.FlagCategory(flag, AppHelpFlagGroups)], flag)
-				}
-			}
-
-			// sort to get a stable ordering
-			sorted := make([]flags.FlagGroup, 0, len(categorized))
-			for cat, flgs := range categorized {
-				sorted = append(sorted, flags.FlagGroup{Name: cat, Flags: flgs})
-			}
-			sort.Sort(flags.ByCategory(sorted))
-
-			// add sorted array to data and render with default printer
-			originalHelpPrinter(w, tmpl, map[string]interface{}{
-				"cmd":              data,
-				"categorizedFlags": sorted,
-			})
-		} else {
-			originalHelpPrinter(w, tmpl, data)
-		}
+		gendocCommand,
 	}
 }
 
@@ -330,7 +271,7 @@ func initializeSecrets(c *cli.Context) error {
 		return err
 	}
 	// Ensure the master key does not yet exist, we're not willing to overwrite
-	configDir := c.GlobalString(configdirFlag.Name)
+	configDir := c.String(configdirFlag.Name)
 	if err := os.Mkdir(configDir, 0700); err != nil && !os.IsExist(err) {
 		return err
 	}
@@ -348,7 +289,7 @@ func initializeSecrets(c *cli.Context) error {
 		return fmt.Errorf("failed to read enough random")
 	}
 	n, p := keystore.StandardScryptN, keystore.StandardScryptP
-	if c.GlobalBool(utils.LightKDFFlag.Name) {
+	if c.Bool(utils.LightKDFFlag.Name) {
 		n, p = keystore.LightScryptN, keystore.LightScryptP
 	}
 	text := "The master seed of clef will be locked with a password.\nPlease specify a password. Do not forget this password!"
@@ -374,7 +315,7 @@ func initializeSecrets(c *cli.Context) error {
 		return fmt.Errorf("master key %v already exists, will not overwrite", location)
 	}
 	// Write the file and print the usual warning message
-	if err = ioutil.WriteFile(location, cipherSeed, 0400); err != nil {
+	if err = os.WriteFile(location, cipherSeed, 0400); err != nil {
 		return err
 	}
 	fmt.Printf("A master seed has been generated into %s\n", location)
@@ -391,8 +332,9 @@ You should treat 'masterseed.json' with utmost secrecy and make a backup of it!
 `)
 	return nil
 }
+
 func attestFile(ctx *cli.Context) error {
-	if len(ctx.Args()) < 1 {
+	if ctx.NArg() < 1 {
 		utils.Fatalf("This command requires an argument.")
 	}
 	if err := initialize(ctx); err != nil {
@@ -403,7 +345,7 @@ func attestFile(ctx *cli.Context) error {
 	if err != nil {
 		utils.Fatalf(err.Error())
 	}
-	configDir := ctx.GlobalString(configdirFlag.Name)
+	configDir := ctx.String(configdirFlag.Name)
 	vaultLocation := filepath.Join(configDir, common.Bytes2Hex(crypto.Keccak256([]byte("vault"), stretchedKey)[:10]))
 	confKey := crypto.Keccak256([]byte("config"), stretchedKey)
 
@@ -416,7 +358,7 @@ func attestFile(ctx *cli.Context) error {
 }
 
 func setCredential(ctx *cli.Context) error {
-	if len(ctx.Args()) < 1 {
+	if ctx.NArg() < 1 {
 		utils.Fatalf("This command requires an address to be passed as an argument")
 	}
 	if err := initialize(ctx); err != nil {
@@ -434,7 +376,7 @@ func setCredential(ctx *cli.Context) error {
 	if err != nil {
 		utils.Fatalf(err.Error())
 	}
-	configDir := ctx.GlobalString(configdirFlag.Name)
+	configDir := ctx.String(configdirFlag.Name)
 	vaultLocation := filepath.Join(configDir, common.Bytes2Hex(crypto.Keccak256([]byte("vault"), stretchedKey)[:10]))
 	pwkey := crypto.Keccak256([]byte("credentials"), stretchedKey)
 
@@ -446,7 +388,7 @@ func setCredential(ctx *cli.Context) error {
 }
 
 func removeCredential(ctx *cli.Context) error {
-	if len(ctx.Args()) < 1 {
+	if ctx.NArg() < 1 {
 		utils.Fatalf("This command requires an address to be passed as an argument")
 	}
 	if err := initialize(ctx); err != nil {
@@ -462,7 +404,7 @@ func removeCredential(ctx *cli.Context) error {
 	if err != nil {
 		utils.Fatalf(err.Error())
 	}
-	configDir := ctx.GlobalString(configdirFlag.Name)
+	configDir := ctx.String(configdirFlag.Name)
 	vaultLocation := filepath.Join(configDir, common.Bytes2Hex(crypto.Keccak256([]byte("vault"), stretchedKey)[:10]))
 	pwkey := crypto.Keccak256([]byte("credentials"), stretchedKey)
 
@@ -482,8 +424,8 @@ func newAccount(c *cli.Context) error {
 	var (
 		ui                        = core.NewCommandlineUI()
 		pwStorage storage.Storage = &storage.NoStorage{}
-		ksLoc                     = c.GlobalString(keystoreFlag.Name)
-		lightKdf                  = c.GlobalBool(utils.LightKDFFlag.Name)
+		ksLoc                     = c.String(keystoreFlag.Name)
+		lightKdf                  = c.Bool(utils.LightKDFFlag.Name)
 	)
 	log.Info("Starting clef", "keystore", ksLoc, "light-kdf", lightKdf)
 	am := core.StartClefAccountManager(ksLoc, true, lightKdf, "")
@@ -501,13 +443,13 @@ func newAccount(c *cli.Context) error {
 func initialize(c *cli.Context) error {
 	// Set up the logger to print everything
 	logOutput := os.Stdout
-	if c.GlobalBool(stdiouiFlag.Name) {
+	if c.Bool(stdiouiFlag.Name) {
 		logOutput = os.Stderr
 		// If using the stdioui, we can't do the 'confirm'-flow
-		if !c.GlobalBool(acceptFlag.Name) {
+		if !c.Bool(acceptFlag.Name) {
 			fmt.Fprint(logOutput, legalWarning)
 		}
-	} else if !c.GlobalBool(acceptFlag.Name) {
+	} else if !c.Bool(acceptFlag.Name) {
 		if !confirm(legalWarning) {
 			return fmt.Errorf("aborted by user")
 		}
@@ -546,8 +488,8 @@ func ipcEndpoint(ipcPath, datadir string) string {
 
 func signer(c *cli.Context) error {
 	// If we have some unrecognized command, bail out
-	if args := c.Args(); len(args) > 0 {
-		return fmt.Errorf("invalid command: %q", args[0])
+	if c.NArg() > 0 {
+		return fmt.Errorf("invalid command: %q", c.Args().First())
 	}
 	if err := initialize(c); err != nil {
 		return err
@@ -555,7 +497,7 @@ func signer(c *cli.Context) error {
 	var (
 		ui core.UIClientAPI
 	)
-	if c.GlobalBool(stdiouiFlag.Name) {
+	if c.Bool(stdiouiFlag.Name) {
 		log.Info("Using stdin/stdout as UI-channel")
 		ui = core.NewStdIOUI()
 	} else {
@@ -563,7 +505,7 @@ func signer(c *cli.Context) error {
 		ui = core.NewCommandlineUI()
 	}
 	// 4bytedb data
-	fourByteLocal := c.GlobalString(customDBFlag.Name)
+	fourByteLocal := c.String(customDBFlag.Name)
 	db, err := fourbyte.NewWithFile(fourByteLocal)
 	if err != nil {
 		utils.Fatalf(err.Error())
@@ -575,7 +517,7 @@ func signer(c *cli.Context) error {
 		api       core.ExternalAPI
 		pwStorage storage.Storage = &storage.NoStorage{}
 	)
-	configDir := c.GlobalString(configdirFlag.Name)
+	configDir := c.String(configdirFlag.Name)
 	if stretchedKey, err := readMasterKey(c, ui); err != nil {
 		log.Warn("Failed to open master, rules disabled", "err", err)
 	} else {
@@ -592,8 +534,8 @@ func signer(c *cli.Context) error {
 		configStorage := storage.NewAESEncryptedStorage(filepath.Join(vaultLocation, "config.json"), confkey)
 
 		// Do we have a rule-file?
-		if ruleFile := c.GlobalString(ruleFlag.Name); ruleFile != "" {
-			ruleJS, err := ioutil.ReadFile(ruleFile)
+		if ruleFile := c.String(ruleFlag.Name); ruleFile != "" {
+			ruleJS, err := os.ReadFile(ruleFile)
 			if err != nil {
 				log.Warn("Could not load rules, disabling", "file", ruleFile, "err", err)
 			} else {
@@ -616,12 +558,12 @@ func signer(c *cli.Context) error {
 		}
 	}
 	var (
-		chainId  = c.GlobalInt64(chainIdFlag.Name)
-		ksLoc    = c.GlobalString(keystoreFlag.Name)
-		lightKdf = c.GlobalBool(utils.LightKDFFlag.Name)
-		advanced = c.GlobalBool(advancedMode.Name)
-		nousb    = c.GlobalBool(utils.NoUSBFlag.Name)
-		scpath   = c.GlobalString(utils.SmartCardDaemonPathFlag.Name)
+		chainId  = c.Int64(chainIdFlag.Name)
+		ksLoc    = c.String(keystoreFlag.Name)
+		lightKdf = c.Bool(utils.LightKDFFlag.Name)
+		advanced = c.Bool(advancedMode.Name)
+		nousb    = c.Bool(utils.NoUSBFlag.Name)
+		scpath   = c.String(utils.SmartCardDaemonPathFlag.Name)
 	)
 	log.Info("Starting signer", "chainid", chainId, "keystore", ksLoc,
 		"light-kdf", lightKdf, "advanced", advanced)
@@ -633,7 +575,7 @@ func signer(c *cli.Context) error {
 	ui.RegisterUIServer(core.NewUIServerAPI(apiImpl))
 	api = apiImpl
 	// Audit logging
-	if logfile := c.GlobalString(auditLogFlag.Name); logfile != "" {
+	if logfile := c.String(auditLogFlag.Name); logfile != "" {
 		api, err = core.NewAuditLogger(logfile, api)
 		if err != nil {
 			utils.Fatalf(err.Error())
@@ -648,26 +590,25 @@ func signer(c *cli.Context) error {
 	rpcAPI := []rpc.API{
 		{
 			Namespace: "account",
-			Public:    true,
 			Service:   api,
 			Version:   "1.0"},
 	}
-	if c.GlobalBool(utils.HTTPEnabledFlag.Name) {
-		vhosts := utils.SplitAndTrim(c.GlobalString(utils.HTTPVirtualHostsFlag.Name))
-		cors := utils.SplitAndTrim(c.GlobalString(utils.HTTPCORSDomainFlag.Name))
+	if c.Bool(utils.HTTPEnabledFlag.Name) {
+		vhosts := utils.SplitAndTrim(c.String(utils.HTTPVirtualHostsFlag.Name))
+		cors := utils.SplitAndTrim(c.String(utils.HTTPCORSDomainFlag.Name))
 
 		srv := rpc.NewServer()
-		err := node.RegisterApis(rpcAPI, []string{"account"}, srv, false)
+		err := node.RegisterApis(rpcAPI, []string{"account"}, srv)
 		if err != nil {
 			utils.Fatalf("Could not register API: %w", err)
 		}
-		handler := node.NewHTTPHandlerStack(srv, cors, vhosts)
+		handler := node.NewHTTPHandlerStack(srv, cors, vhosts, nil)
 
 		// set port
 		port := c.Int(rpcPortFlag.Name)
 
 		// start http server
-		httpEndpoint := fmt.Sprintf("%s:%d", c.GlobalString(utils.HTTPListenAddrFlag.Name), port)
+		httpEndpoint := fmt.Sprintf("%s:%d", c.String(utils.HTTPListenAddrFlag.Name), port)
 		httpServer, addr, err := node.StartHTTPEndpoint(httpEndpoint, rpc.DefaultHTTPTimeouts, handler)
 		if err != nil {
 			utils.Fatalf("Could not start RPC api: %v", err)
@@ -681,8 +622,8 @@ func signer(c *cli.Context) error {
 			log.Info("HTTP endpoint closed", "url", extapiURL)
 		}()
 	}
-	if !c.GlobalBool(utils.IPCDisabledFlag.Name) {
-		givenPath := c.GlobalString(utils.IPCPathFlag.Name)
+	if !c.Bool(utils.IPCDisabledFlag.Name) {
+		givenPath := c.String(utils.IPCPathFlag.Name)
 		ipcapiURL = ipcEndpoint(filepath.Join(givenPath, "clef.ipc"), configDir)
 		listener, _, err := rpc.StartIPCEndpoint(ipcapiURL, rpcAPI)
 		if err != nil {
@@ -695,7 +636,7 @@ func signer(c *cli.Context) error {
 		}()
 	}
 
-	if c.GlobalBool(testFlag.Name) {
+	if c.Bool(testFlag.Name) {
 		log.Info("Performing UI test")
 		go testExternalUI(apiImpl)
 	}
@@ -721,7 +662,7 @@ func signer(c *cli.Context) error {
 // persistence requirements.
 func DefaultConfigDir() string {
 	// Try to place the data folder in the user's home dir
-	home := utils.HomeDir()
+	home := flags.HomeDir()
 	if home != "" {
 		if runtime.GOOS == "darwin" {
 			return filepath.Join(home, "Library", "Signer")
@@ -741,17 +682,17 @@ func DefaultConfigDir() string {
 func readMasterKey(ctx *cli.Context, ui core.UIClientAPI) ([]byte, error) {
 	var (
 		file      string
-		configDir = ctx.GlobalString(configdirFlag.Name)
+		configDir = ctx.String(configdirFlag.Name)
 	)
-	if ctx.GlobalIsSet(signerSecretFlag.Name) {
-		file = ctx.GlobalString(signerSecretFlag.Name)
+	if ctx.IsSet(signerSecretFlag.Name) {
+		file = ctx.String(signerSecretFlag.Name)
 	} else {
 		file = filepath.Join(configDir, "masterseed.json")
 	}
 	if err := checkFile(file); err != nil {
 		return nil, err
 	}
-	cipherKey, err := ioutil.ReadFile(file)
+	cipherKey, err := os.ReadFile(file)
 	if err != nil {
 		return nil, err
 	}
@@ -898,7 +839,7 @@ func testExternalUI(api *core.SignerAPI) {
 		addr, _ := common.NewMixedcaseAddressFromString("0x0011223344556677889900112233445566778899")
 		data := `{"types":{"EIP712Domain":[{"name":"name","type":"string"},{"name":"version","type":"string"},{"name":"chainId","type":"uint256"},{"name":"verifyingContract","type":"address"}],"Person":[{"name":"name","type":"string"},{"name":"test","type":"uint8"},{"name":"wallet","type":"address"}],"Mail":[{"name":"from","type":"Person"},{"name":"to","type":"Person"},{"name":"contents","type":"string"}]},"primaryType":"Mail","domain":{"name":"Ether Mail","version":"1","chainId":"1","verifyingContract":"0xCCCcccccCCCCcCCCCCCcCcCccCcCCCcCcccccccC"},"message":{"from":{"name":"Cow","test":"3","wallet":"0xcD2a3d9F938E13CD947Ec05AbC7FE734Df8DD826"},"to":{"name":"Bob","wallet":"0xbBbBBBBbbBBBbbbBbbBbbbbBBbBbbbbBbBbbBBbB","test":"2"},"contents":"Hello, Bob!"}}`
 		//_, err := api.SignData(ctx, accounts.MimetypeTypedData, *addr, hexutil.Encode([]byte(data)))
-		var typedData core.TypedData
+		var typedData apitypes.TypedData
 		json.Unmarshal([]byte(data), &typedData)
 		_, err := api.SignTypedData(ctx, *addr, typedData)
 		expectApprove("sign 712 typed data", err)
@@ -997,7 +938,7 @@ func decryptSeed(keyjson []byte, auth string) ([]byte, error) {
 }
 
 // GenDoc outputs examples of all structures used in json-rpc communication
-func GenDoc(ctx *cli.Context) {
+func GenDoc(ctx *cli.Context) error {
 
 	var (
 		a    = common.HexToAddress("0xdeadbeef000000000000000000000000deadbeef")
@@ -1025,7 +966,7 @@ func GenDoc(ctx *cli.Context) {
 			"of the work in canonicalizing and making sense of the data, and it's up to the UI to present" +
 			"the user with the contents of the `message`"
 		sighash, msg := accounts.TextAndHash([]byte("hello world"))
-		messages := []*core.NameValueType{{Name: "message", Value: msg, Typ: accounts.MimetypeTextPlain}}
+		messages := []*apitypes.NameValueType{{Name: "message", Value: msg, Typ: accounts.MimetypeTextPlain}}
 
 		add("SignDataRequest", desc, &core.SignDataRequest{
 			Address:     common.NewMixedcaseAddress(a),
@@ -1147,4 +1088,5 @@ These data types are defined in the channel between clef and the UI`)
 	for _, elem := range output {
 		fmt.Println(elem)
 	}
+	return nil
 }
