@@ -22,7 +22,6 @@ package signify
 import (
 	"bufio"
 	"fmt"
-	"io/ioutil"
 	"log"
 	"os"
 	"os/exec"
@@ -35,7 +34,7 @@ func Fuzz(data []byte) int {
 	if len(data) < 32 {
 		return -1
 	}
-	tmpFile, err := ioutil.TempFile("", "")
+	tmpFile, err := os.CreateTemp("", "")
 	if err != nil {
 		panic(err)
 	}
@@ -76,7 +75,7 @@ func Fuzz(data []byte) int {
 
 	// Write the public key into the file to pass it as
 	// an argument to signify-openbsd
-	pubKeyFile, err := ioutil.TempFile("", "")
+	pubKeyFile, err := os.CreateTemp("", "")
 	if err != nil {
 		panic(err)
 	}
@@ -128,7 +127,7 @@ func getKey(fileS string) (string, error) {
 
 func createKeyPair() (string, string) {
 	// Create key and put it in correct format
-	tmpKey, err := ioutil.TempFile("", "")
+	tmpKey, err := os.CreateTemp("", "")
 	if err != nil {
 		panic(err)
 	}
