@@ -99,8 +99,8 @@ func MakeProtocols(backend Backend, dnsdisc enode.Iterator) []p2p.Protocol {
 			Version: version,
 			Length:  protocolLengths[version],
 			Run: func(p *p2p.Peer, rw p2p.MsgReadWriter) error {
-				return backend.RunPeer(newPeer(version, p, rw), func(peer *Peer) error {
-					return handle(backend, peer)
+				return backend.RunPeer(NewPeer(version, p, rw), func(peer *Peer) error {
+					return Handle(backend, peer)
 				})
 			},
 			NodeInfo: func() interface{} {
@@ -116,21 +116,21 @@ func MakeProtocols(backend Backend, dnsdisc enode.Iterator) []p2p.Protocol {
 	return protocols
 }
 
-// handle is the callback invoked to manage the life cycle of a `snap` peer.
+// Handle is the callback invoked to manage the life cycle of a `snap` peer.
 // When this function terminates, the peer is disconnected.
-func handle(backend Backend, peer *Peer) error {
+func Handle(backend Backend, peer *Peer) error {
 	for {
-		if err := handleMessage(backend, peer); err != nil {
+		if err := HandleMessage(backend, peer); err != nil {
 			peer.Log().Debug("Message handling failed in `snap`", "err", err)
 			return err
 		}
 	}
 }
 
-// handleMessage is invoked whenever an inbound message is received from a
+// HandleMessage is invoked whenever an inbound message is received from a
 // remote peer on the `snap` protocol. The remote connection is torn down upon
 // returning any error.
-func handleMessage(backend Backend, peer *Peer) error {
+func HandleMessage(backend Backend, peer *Peer) error {
 	// Read the next message from the remote peer, and ensure it's fully consumed
 	msg, err := peer.rw.ReadMsg()
 	if err != nil {
@@ -161,60 +161,10 @@ func handleMessage(backend Backend, peer *Peer) error {
 		if err := msg.Decode(&req); err != nil {
 			return fmt.Errorf("%w: message %v: %v", errDecode, msg, err)
 		}
-		if req.Bytes > softResponseLimit {
-			req.Bytes = softResponseLimit
-		}
-		// Retrieve the requested state and bail out if non existent
-		tr, err := trie.New(req.Root, backend.Chain().StateCache().TrieDB())
-		if err != nil {
-			return p2p.Send(peer.rw, AccountRangeMsg, &AccountRangePacket{ID: req.ID})
-		}
-		it, err := backend.Chain().Snapshots().AccountIterator(req.Root, req.Origin)
-		if err != nil {
-			return p2p.Send(peer.rw, AccountRangeMsg, &AccountRangePacket{ID: req.ID})
-		}
-		// Iterate over the requested range and pile accounts up
-		var (
-			accounts []*AccountData
-			size     uint64
-			last     common.Hash
-		)
-		for it.Next() && size < req.Bytes {
-			hash, account := it.Hash(), common.CopyBytes(it.Account())
+		// Service the request, potentially returning nothing in case of errors
+		accounts, proofs := ServiceGetAccountRangeQuery(backend.Chain(), &req)
 
-			// Track the returned interval for the Merkle proofs
-			last = hash
-
-			// Assemble the reply item
-			size += uint64(common.HashLength + len(account))
-			accounts = append(accounts, &AccountData{
-				Hash: hash,
-				Body: account,
-			})
-			// If we've exceeded the request threshold, abort
-			if bytes.Compare(hash[:], req.Limit[:]) >= 0 {
-				break
-			}
-		}
-		it.Release()
-
-		// Generate the Merkle proofs for the first and last account
-		proof := light.NewNodeSet()
-		if err := tr.Prove(req.Origin[:], 0, proof); err != nil {
-			log.Warn("Failed to prove account range", "origin", req.Origin, "err", err)
-			return p2p.Send(peer.rw, AccountRangeMsg, &AccountRangePacket{ID: req.ID})
-		}
-		if last != (common.Hash{}) {
-			if err := tr.Prove(last[:], 0, proof); err != nil {
-				log.Warn("Failed to prove account range", "last", last, "err", err)
-				return p2p.Send(peer.rw, AccountRangeMsg, &AccountRangePacket{ID: req.ID})
-			}
-		}
-		var proofs [][]byte
-		for _, blob := range proof.NodeList() {
-			proofs = append(proofs, blob)
-		}
-		// Send back anything accumulated
+		// Send back anything accumulated (or empty in case of errors)
 		return p2p.Send(peer.rw, AccountRangeMsg, &AccountRangePacket{
 			ID:       req.ID,
 			Accounts: accounts,
@@ -243,111 +193,10 @@ func handleMessage(backend Backend, peer *Peer) error {
 		if err := msg.Decode(&req); err != nil {
 			return fmt.Errorf("%w: message %v: %v", errDecode, msg, err)
 		}
-		if req.Bytes > softResponseLimit {
-			req.Bytes = softResponseLimit
-		}
-		// TODO(karalabe): Do we want to enforce > 0 accounts and 1 account if origin is set?
-		// TODO(karalabe):   - Logging locally is not ideal as remote faulst annoy the local user
-		// TODO(karalabe):   - Dropping the remote peer is less flexible wrt client bugs (slow is better than non-functional)
+		// Service the request, potentially returning nothing in case of errors
+		slots, proofs := ServiceGetStorageRangesQuery(backend.Chain(), &req)
 
-		// Calculate the hard limit at which to abort, even if mid storage trie
-		hardLimit := uint64(float64(req.Bytes) * (1 + stateLookupSlack))
-
-		// Retrieve storage ranges until the packet limit is reached
-		var (
-			slots  [][]*StorageData
-			proofs [][]byte
-			size   uint64
-		)
-		for _, account := range req.Accounts {
-			// If we've exceeded the requested data limit, abort without opening
-			// a new storage range (that we'd need to prove due to exceeded size)
-			if size >= req.Bytes {
-				break
-			}
-			// The first account might start from a different origin and end sooner
-			var origin common.Hash
-			if len(req.Origin) > 0 {
-				origin, req.Origin = common.BytesToHash(req.Origin), nil
-			}
-			var limit = common.HexToHash("0xffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff")
-			if len(req.Limit) > 0 {
-				limit, req.Limit = common.BytesToHash(req.Limit), nil
-			}
-			// Retrieve the requested state and bail out if non existent
-			it, err := backend.Chain().Snapshots().StorageIterator(req.Root, account, origin)
-			if err != nil {
-				return p2p.Send(peer.rw, StorageRangesMsg, &StorageRangesPacket{ID: req.ID})
-			}
-			// Iterate over the requested range and pile slots up
-			var (
-				storage []*StorageData
-				last    common.Hash
-				abort   bool
-			)
-			for it.Next() {
-				if size >= hardLimit {
-					abort = true
-					break
-				}
-				hash, slot := it.Hash(), common.CopyBytes(it.Slot())
-
-				// Track the returned interval for the Merkle proofs
-				last = hash
-
-				// Assemble the reply item
-				size += uint64(common.HashLength + len(slot))
-				storage = append(storage, &StorageData{
-					Hash: hash,
-					Body: slot,
-				})
-				// If we've exceeded the request threshold, abort
-				if bytes.Compare(hash[:], limit[:]) >= 0 {
-					break
-				}
-			}
-			slots = append(slots, storage)
-			it.Release()
-
-			// Generate the Merkle proofs for the first and last storage slot, but
-			// only if the response was capped. If the entire storage trie included
-			// in the response, no need for any proofs.
-			if origin != (common.Hash{}) || abort {
-				// Request started at a non-zero hash or was capped prematurely, add
-				// the endpoint Merkle proofs
-				accTrie, err := trie.New(req.Root, backend.Chain().StateCache().TrieDB())
-				if err != nil {
-					return p2p.Send(peer.rw, StorageRangesMsg, &StorageRangesPacket{ID: req.ID})
-				}
-				var acc types.StateAccount
-				if err := rlp.DecodeBytes(accTrie.Get(account[:]), &acc); err != nil {
-					return p2p.Send(peer.rw, StorageRangesMsg, &StorageRangesPacket{ID: req.ID})
-				}
-				stTrie, err := trie.New(acc.Root, backend.Chain().StateCache().TrieDB())
-				if err != nil {
-					return p2p.Send(peer.rw, StorageRangesMsg, &StorageRangesPacket{ID: req.ID})
-				}
-				proof := light.NewNodeSet()
-				if err := stTrie.Prove(origin[:], 0, proof); err != nil {
-					log.Warn("Failed to prove storage range", "origin", req.Origin, "err", err)
-					return p2p.Send(peer.rw, StorageRangesMsg, &StorageRangesPacket{ID: req.ID})
-				}
-				if last != (common.Hash{}) {
-					if err := stTrie.Prove(last[:], 0, proof); err != nil {
-						log.Warn("Failed to prove storage range", "last", last, "err", err)
-						return p2p.Send(peer.rw, StorageRangesMsg, &StorageRangesPacket{ID: req.ID})
-					}
-				}
-				for _, blob := range proof.NodeList() {
-					proofs = append(proofs, blob)
-				}
-				// Proof terminates the reply as proofs are only added if a node
-				// refuses to serve more data (exception when a contract fetch is
-				// finishing, but that's that).
-				break
-			}
-		}
-		// Send back anything accumulated
+		// Send back anything accumulated (or empty in case of errors)
 		return p2p.Send(peer.rw, StorageRangesMsg, &StorageRangesPacket{
 			ID:    req.ID,
 			Slots: slots,
@@ -378,31 +227,10 @@ func handleMessage(backend Backend, peer *Peer) error {
 		if err := msg.Decode(&req); err != nil {
 			return fmt.Errorf("%w: message %v: %v", errDecode, msg, err)
 		}
-		if req.Bytes > softResponseLimit {
-			req.Bytes = softResponseLimit
-		}
-		if len(req.Hashes) > maxCodeLookups {
-			req.Hashes = req.Hashes[:maxCodeLookups]
-		}
-		// Retrieve bytecodes until the packet size limit is reached
-		var (
-			codes [][]byte
-			bytes uint64
-		)
-		for _, hash := range req.Hashes {
-			if hash == emptyCode {
-				// Peers should not request the empty code, but if they do, at
-				// least sent them back a correct response without db lookups
-				codes = append(codes, []byte{})
-			} else if blob, err := backend.Chain().ContractCode(hash); err == nil {
-				codes = append(codes, blob)
-				bytes += uint64(len(blob))
-			}
-			if bytes > req.Bytes {
-				break
-			}
-		}
-		// Send back anything accumulated
+		// Service the request, potentially returning nothing in case of errors
+		codes := ServiceGetByteCodesQuery(backend.Chain(), &req)
+
+		// Send back anything accumulated (or empty in case of errors)
 		return p2p.Send(peer.rw, ByteCodesMsg, &ByteCodesPacket{
 			ID:    req.ID,
 			Codes: codes,
@@ -424,80 +252,12 @@ func handleMessage(backend Backend, peer *Peer) error {
 		if err := msg.Decode(&req); err != nil {
 			return fmt.Errorf("%w: message %v: %v", errDecode, msg, err)
 		}
-		if req.Bytes > softResponseLimit {
-			req.Bytes = softResponseLimit
-		}
-		// Make sure we have the state associated with the request
-		triedb := backend.Chain().StateCache().TrieDB()
-
-		accTrie, err := trie.NewSecure(req.Root, triedb)
+		// Service the request, potentially returning nothing in case of errors
+		nodes, err := ServiceGetTrieNodesQuery(backend.Chain(), &req, start)
 		if err != nil {
-			// We don't have the requested state available, bail out
-			return p2p.Send(peer.rw, TrieNodesMsg, &TrieNodesPacket{ID: req.ID})
+			return err
 		}
-		snap := backend.Chain().Snapshots().Snapshot(req.Root)
-		if snap == nil {
-			// We don't have the requested state snapshotted yet, bail out.
-			// In reality we could still serve using the account and storage
-			// tries only, but let's protect the node a bit while it's doing
-			// snapshot generation.
-			return p2p.Send(peer.rw, TrieNodesMsg, &TrieNodesPacket{ID: req.ID})
-		}
-		// Retrieve trie nodes until the packet size limit is reached
-		var (
-			nodes [][]byte
-			bytes uint64
-			loads int // Trie hash expansions to cound database reads
-		)
-		for _, pathset := range req.Paths {
-			switch len(pathset) {
-			case 0:
-				// Ensure we penalize invalid requests
-				return fmt.Errorf("%w: zero-item pathset requested", errBadRequest)
-
-			case 1:
-				// If we're only retrieving an account trie node, fetch it directly
-				blob, resolved, err := accTrie.TryGetNode(pathset[0])
-				loads += resolved // always account database reads, even for failures
-				if err != nil {
-					break
-				}
-				nodes = append(nodes, blob)
-				bytes += uint64(len(blob))
-
-			default:
-				// Storage slots requested, open the storage trie and retrieve from there
-				account, err := snap.Account(common.BytesToHash(pathset[0]))
-				loads++ // always account database reads, even for failures
-				if err != nil || account == nil {
-					break
-				}
-				stTrie, err := trie.NewSecure(common.BytesToHash(account.Root), triedb)
-				loads++ // always account database reads, even for failures
-				if err != nil {
-					break
-				}
-				for _, path := range pathset[1:] {
-					blob, resolved, err := stTrie.TryGetNode(path)
-					loads += resolved // always account database reads, even for failures
-					if err != nil {
-						break
-					}
-					nodes = append(nodes, blob)
-					bytes += uint64(len(blob))
-
-					// Sanity check limits to avoid DoS on the store trie loads
-					if bytes > req.Bytes || loads > maxTrieNodeLookups || time.Since(start) > maxTrieNodeTimeSpent {
-						break
-					}
-				}
-			}
-			// Abort request processing if we've exceeded our limits
-			if bytes > req.Bytes || loads > maxTrieNodeLookups || time.Since(start) > maxTrieNodeTimeSpent {
-				break
-			}
-		}
-		// Send back anything accumulated
+		// Send back anything accumulated (or empty in case of errors)
 		return p2p.Send(peer.rw, TrieNodesMsg, &TrieNodesPacket{
 			ID:    req.ID,
 			Nodes: nodes,
@@ -518,6 +278,287 @@ func handleMessage(backend Backend, peer *Peer) error {
 	}
 }
 
+// ServiceGetAccountRangeQuery assembles the response to an account range query.
+// It is exposed to allow external packages to test protocol behavior.
+func ServiceGetAccountRangeQuery(chain *core.BlockChain, req *GetAccountRangePacket) ([]*AccountData, [][]byte) {
+	if req.Bytes > softResponseLimit {
+		req.Bytes = softResponseLimit
+	}
+	// Retrieve the requested state and bail out if non existent
+	tr, err := trie.New(common.Hash{}, req.Root, chain.StateCache().TrieDB())
+	if err != nil {
+		return nil, nil
+	}
+	it, err := chain.Snapshots().AccountIterator(req.Root, req.Origin)
+	if err != nil {
+		return nil, nil
+	}
+	// Iterate over the requested range and pile accounts up
+	var (
+		accounts []*AccountData
+		size     uint64
+		last     common.Hash
+	)
+	for it.Next() {
+		hash, account := it.Hash(), common.CopyBytes(it.Account())
+
+		// Track the returned interval for the Merkle proofs
+		last = hash
+
+		// Assemble the reply item
+		size += uint64(common.HashLength + len(account))
+		accounts = append(accounts, &AccountData{
+			Hash: hash,
+			Body: account,
+		})
+		// If we've exceeded the request threshold, abort
+		if bytes.Compare(hash[:], req.Limit[:]) >= 0 {
+			break
+		}
+		if size > req.Bytes {
+			break
+		}
+	}
+	it.Release()
+
+	// Generate the Merkle proofs for the first and last account
+	proof := light.NewNodeSet()
+	if err := tr.Prove(req.Origin[:], 0, proof); err != nil {
+		log.Warn("Failed to prove account range", "origin", req.Origin, "err", err)
+		return nil, nil
+	}
+	if last != (common.Hash{}) {
+		if err := tr.Prove(last[:], 0, proof); err != nil {
+			log.Warn("Failed to prove account range", "last", last, "err", err)
+			return nil, nil
+		}
+	}
+	var proofs [][]byte
+	for _, blob := range proof.NodeList() {
+		proofs = append(proofs, blob)
+	}
+	return accounts, proofs
+}
+
+func ServiceGetStorageRangesQuery(chain *core.BlockChain, req *GetStorageRangesPacket) ([][]*StorageData, [][]byte) {
+	if req.Bytes > softResponseLimit {
+		req.Bytes = softResponseLimit
+	}
+	// TODO(karalabe): Do we want to enforce > 0 accounts and 1 account if origin is set?
+	// TODO(karalabe):   - Logging locally is not ideal as remote faulst annoy the local user
+	// TODO(karalabe):   - Dropping the remote peer is less flexible wrt client bugs (slow is better than non-functional)
+
+	// Calculate the hard limit at which to abort, even if mid storage trie
+	hardLimit := uint64(float64(req.Bytes) * (1 + stateLookupSlack))
+
+	// Retrieve storage ranges until the packet limit is reached
+	var (
+		slots  [][]*StorageData
+		proofs [][]byte
+		size   uint64
+	)
+	for _, account := range req.Accounts {
+		// If we've exceeded the requested data limit, abort without opening
+		// a new storage range (that we'd need to prove due to exceeded size)
+		if size >= req.Bytes {
+			break
+		}
+		// The first account might start from a different origin and end sooner
+		var origin common.Hash
+		if len(req.Origin) > 0 {
+			origin, req.Origin = common.BytesToHash(req.Origin), nil
+		}
+		var limit = common.HexToHash("0xffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff")
+		if len(req.Limit) > 0 {
+			limit, req.Limit = common.BytesToHash(req.Limit), nil
+		}
+		// Retrieve the requested state and bail out if non existent
+		it, err := chain.Snapshots().StorageIterator(req.Root, account, origin)
+		if err != nil {
+			return nil, nil
+		}
+		// Iterate over the requested range and pile slots up
+		var (
+			storage []*StorageData
+			last    common.Hash
+			abort   bool
+		)
+		for it.Next() {
+			if size >= hardLimit {
+				abort = true
+				break
+			}
+			hash, slot := it.Hash(), common.CopyBytes(it.Slot())
+
+			// Track the returned interval for the Merkle proofs
+			last = hash
+
+			// Assemble the reply item
+			size += uint64(common.HashLength + len(slot))
+			storage = append(storage, &StorageData{
+				Hash: hash,
+				Body: slot,
+			})
+			// If we've exceeded the request threshold, abort
+			if bytes.Compare(hash[:], limit[:]) >= 0 {
+				break
+			}
+		}
+		if len(storage) > 0 {
+			slots = append(slots, storage)
+		}
+		it.Release()
+
+		// Generate the Merkle proofs for the first and last storage slot, but
+		// only if the response was capped. If the entire storage trie included
+		// in the response, no need for any proofs.
+		if origin != (common.Hash{}) || (abort && len(storage) > 0) {
+			// Request started at a non-zero hash or was capped prematurely, add
+			// the endpoint Merkle proofs
+			accTrie, err := trie.New(common.Hash{}, req.Root, chain.StateCache().TrieDB())
+			if err != nil {
+				return nil, nil
+			}
+			var acc types.StateAccount
+			if err := rlp.DecodeBytes(accTrie.Get(account[:]), &acc); err != nil {
+				return nil, nil
+			}
+			stTrie, err := trie.New(account, acc.Root, chain.StateCache().TrieDB())
+			if err != nil {
+				return nil, nil
+			}
+			proof := light.NewNodeSet()
+			if err := stTrie.Prove(origin[:], 0, proof); err != nil {
+				log.Warn("Failed to prove storage range", "origin", req.Origin, "err", err)
+				return nil, nil
+			}
+			if last != (common.Hash{}) {
+				if err := stTrie.Prove(last[:], 0, proof); err != nil {
+					log.Warn("Failed to prove storage range", "last", last, "err", err)
+					return nil, nil
+				}
+			}
+			for _, blob := range proof.NodeList() {
+				proofs = append(proofs, blob)
+			}
+			// Proof terminates the reply as proofs are only added if a node
+			// refuses to serve more data (exception when a contract fetch is
+			// finishing, but that's that).
+			break
+		}
+	}
+	return slots, proofs
+}
+
+// ServiceGetByteCodesQuery assembles the response to a byte codes query.
+// It is exposed to allow external packages to test protocol behavior.
+func ServiceGetByteCodesQuery(chain *core.BlockChain, req *GetByteCodesPacket) [][]byte {
+	if req.Bytes > softResponseLimit {
+		req.Bytes = softResponseLimit
+	}
+	if len(req.Hashes) > maxCodeLookups {
+		req.Hashes = req.Hashes[:maxCodeLookups]
+	}
+	// Retrieve bytecodes until the packet size limit is reached
+	var (
+		codes [][]byte
+		bytes uint64
+	)
+	for _, hash := range req.Hashes {
+		if hash == emptyCode {
+			// Peers should not request the empty code, but if they do, at
+			// least sent them back a correct response without db lookups
+			codes = append(codes, []byte{})
+		} else if blob, err := chain.ContractCodeWithPrefix(hash); err == nil {
+			codes = append(codes, blob)
+			bytes += uint64(len(blob))
+		}
+		if bytes > req.Bytes {
+			break
+		}
+	}
+	return codes
+}
+
+// ServiceGetTrieNodesQuery assembles the response to a trie nodes query.
+// It is exposed to allow external packages to test protocol behavior.
+func ServiceGetTrieNodesQuery(chain *core.BlockChain, req *GetTrieNodesPacket, start time.Time) ([][]byte, error) {
+	if req.Bytes > softResponseLimit {
+		req.Bytes = softResponseLimit
+	}
+	// Make sure we have the state associated with the request
+	triedb := chain.StateCache().TrieDB()
+
+	accTrie, err := trie.NewSecure(common.Hash{}, req.Root, triedb)
+	if err != nil {
+		// We don't have the requested state available, bail out
+		return nil, nil
+	}
+	snap := chain.Snapshots().Snapshot(req.Root)
+	if snap == nil {
+		// We don't have the requested state snapshotted yet, bail out.
+		// In reality we could still serve using the account and storage
+		// tries only, but let's protect the node a bit while it's doing
+		// snapshot generation.
+		return nil, nil
+	}
+	// Retrieve trie nodes until the packet size limit is reached
+	var (
+		nodes [][]byte
+		bytes uint64
+		loads int // Trie hash expansions to cound database reads
+	)
+	for _, pathset := range req.Paths {
+		switch len(pathset) {
+		case 0:
+			// Ensure we penalize invalid requests
+			return nil, fmt.Errorf("%w: zero-item pathset requested", errBadRequest)
+
+		case 1:
+			// If we're only retrieving an account trie node, fetch it directly
+			blob, resolved, err := accTrie.TryGetNode(pathset[0])
+			loads += resolved // always account database reads, even for failures
+			if err != nil {
+				break
+			}
+			nodes = append(nodes, blob)
+			bytes += uint64(len(blob))
+
+		default:
+			// Storage slots requested, open the storage trie and retrieve from there
+			account, err := snap.Account(common.BytesToHash(pathset[0]))
+			loads++ // always account database reads, even for failures
+			if err != nil || account == nil {
+				break
+			}
+			stTrie, err := trie.NewSecure(common.BytesToHash(pathset[0]), common.BytesToHash(account.Root), triedb)
+			loads++ // always account database reads, even for failures
+			if err != nil {
+				break
+			}
+			for _, path := range pathset[1:] {
+				blob, resolved, err := stTrie.TryGetNode(path)
+				loads += resolved // always account database reads, even for failures
+				if err != nil {
+					break
+				}
+				nodes = append(nodes, blob)
+				bytes += uint64(len(blob))
+
+				// Sanity check limits to avoid DoS on the store trie loads
+				if bytes > req.Bytes || loads > maxTrieNodeLookups || time.Since(start) > maxTrieNodeTimeSpent {
+					break
+				}
+			}
+		}
+		// Abort request processing if we've exceeded our limits
+		if bytes > req.Bytes || loads > maxTrieNodeLookups || time.Since(start) > maxTrieNodeTimeSpent {
+			break
+		}
+	}
+	return nodes, nil
+}
+
 // NodeInfo represents a short summary of the `snap` sub-protocol metadata
 // known about the host peer.
 type NodeInfo struct{}
