@@ -69,7 +69,7 @@ const schema string = `
         transaction: Transaction!
     }
 
-    #EIP-2718 
+    #EIP-2718
     type AccessTuple{
         address: Address!
         storageKeys : [Bytes32!]!
@@ -94,10 +94,12 @@ const schema string = `
         value: BigInt!
         # GasPrice is the price offered to miners for gas, in wei per unit.
         gasPrice: BigInt!
-        # MaxFeePerGas is the maximum fee per gas offered to include a transaction, in wei. 
-		maxFeePerGas: BigInt
-        # MaxPriorityFeePerGas is the maximum miner tip per gas offered to include a transaction, in wei. 
-		maxPriorityFeePerGas: BigInt
+        # MaxFeePerGas is the maximum fee per gas offered to include a transaction, in wei.
+        maxFeePerGas: BigInt
+        # MaxPriorityFeePerGas is the maximum miner tip per gas offered to include a transaction, in wei.
+        maxPriorityFeePerGas: BigInt
+        # EffectiveTip is the actual amount of reward going to miner after considering the max fee cap.
+        effectiveTip: BigInt
         # Gas is the maximum amount of gas this transaction can consume.
         gas: Long!
         # InputData is the data supplied to the target of the transaction.
@@ -135,9 +137,16 @@ const schema string = `
         r: BigInt!
         s: BigInt!
         v: BigInt!
-        #Envelope transaction support
+        # Envelope transaction support
         type: Int
         accessList: [AccessTuple!]
+        # Raw is the canonical encoding of the transaction.
+        # For legacy transactions, it returns the RLP encoding.
+        # For EIP-2718 typed transactions, it returns the type and payload.
+        raw: Bytes!
+        # RawReceipt is the canonical encoding of the receipt. For post EIP-2718 typed transactions
+        # this is equivalent to TxType || ReceiptEncoding.
+        rawReceipt: Bytes!
     }
 
     # BlockFilterCriteria encapsulates log filter criteria for a filter applied
@@ -187,8 +196,10 @@ const schema string = `
         gasLimit: Long!
         # GasUsed is the amount of gas that was used executing transactions in this block.
         gasUsed: Long!
-        # BaseFeePerGas is the fee perunit of gas burned by the protocol in this block.
-		baseFeePerGas: BigInt
+        # BaseFeePerGas is the fee per unit of gas burned by the protocol in this block.
+        baseFeePerGas: BigInt
+        # NextBaseFeePerGas is the fee per unit of gas which needs to be burned in the next block.
+        nextBaseFeePerGas: BigInt
         # Timestamp is the unix timestamp at which this block was mined.
         timestamp: Long!
         # LogsBloom is a bloom filter that can be used to check if a block may
@@ -231,6 +242,10 @@ const schema string = `
         # EstimateGas estimates the amount of gas that will be required for
         # successful execution of a transaction at the current block's state.
         estimateGas(data: CallData!): Long!
+        # RawHeader is the RLP encoding of the block's header.
+        rawHeader: Bytes!
+        # Raw is the RLP encoding of the block.
+        raw: Bytes!
     }
 
     # CallData represents the data associated with a local contract call.
@@ -244,10 +259,10 @@ const schema string = `
         gas: Long
         # GasPrice is the price, in wei, offered for each unit of gas.
         gasPrice: BigInt
-        # MaxFeePerGas is the maximum fee per gas offered, in wei. 
-		maxFeePerGas: BigInt
-        # MaxPriorityFeePerGas is the maximum miner tip per gas offered, in wei. 
-		maxPriorityFeePerGas: BigInt
+        # MaxFeePerGas is the maximum fee per gas offered, in wei.
+        maxFeePerGas: BigInt
+        # MaxPriorityFeePerGas is the maximum miner tip per gas offered, in wei.
+        maxPriorityFeePerGas: BigInt
         # Value is the value, in wei, sent along with the call.
         value: BigInt
         # Data is the data sent to the callee.
@@ -297,12 +312,6 @@ const schema string = `
         currentBlock: Long!
         # HighestBlock is the latest known block number.
         highestBlock: Long!
-        # PulledStates is the number of state entries fetched so far, or null
-        # if this is not known or not relevant.
-        pulledStates: Long
-        # KnownStates is the number of states the node knows of so far, or null
-        # if this is not known or not relevant.
-        knownStates: Long
     }
 
     # Pending represents the current pending state.
